package de.mattv.fileserver.security;

import de.mattv.fileserver.data.Token;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.util.NonNull;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.lang.Nullable;
import org.springframework.scheduling.annotation.Scheduled;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;

public class TokenService {
    private static final ConcurrentHashMap<String, Token> TOKENS = new ConcurrentHashMap<>();
    public static final int TOKEN_LENGTH = 32;
    public static final int SHARE_TOKEN_LENGTH = 30;

    @Scheduled(fixedRate = 2, timeUnit = TimeUnit.HOURS)
    private static void cleanExpiredTokens() {
        TOKENS.entrySet().removeIf(entry -> entry.getValue().expired());
    }

    public static String createToken(@NonNull User user) {
        String token = RandomStringUtils.random(TOKEN_LENGTH, true, true);
        TOKENS.put(token, new Token(token, user, null));
        return token;
    }

    public static @Nullable Token getToken(@NonNull String token) {
        if (token.length() == TOKEN_LENGTH) {
            Token found = TOKENS.get(token);
            if (found == null) return null;
            if (found.expired()) {
                TOKENS.remove(token);
                return null;
            }
            found.refresh();
            return found;
        } else if (token.length() == SHARE_TOKEN_LENGTH) {
            var share = ShareService.getShare(token);
            if (share == null) return null;
            return new Token(token, share.getLeft(), share.getRight().id);
        } else return null;
    }

    public static void logout(@NonNull Token token) {
        TOKENS.remove(token.getToken());
    }

    public static void logoutAll(long id) {
        TOKENS.entrySet().removeIf(entry -> {
            Token token = entry.getValue();
            return token.getUser().id == id || token.getRealUser().id == id;
        });
    }
}
